<?php

/*
 * AfterLogic WebMail Pro PHP by AfterLogic Corp. <support@afterlogic.com>
 *
 * Copyright (C) 2002-2009  AfterLogic Corp. (www.afterlogic.com)
 * Distributed under the terms of the proprietary AfterLogic Computer License (LICENSE.txt)
 * 
 */

	if (!defined('WM_ROOTPATH')) define('WM_ROOTPATH', (dirname(__FILE__).'/../'));
	
	require_once(WM_ROOTPATH.'class_settings.php');

	define ('LOG_FORMAT_OPTIONS_NONE', 0);		// Use default formatting.
	define ('LOG_FORMAT_OPTIONS_ADD_DATE', 1);	// Include the current date in the timestamp.
	
	define('LOG_FILENAME', 'log_'.date('Y-m-d').'.txt');
	define('LOG_LINELIMIT', 8000);
	define('LOG_TIMERUN', 200);
	
	class CLog
	{
		/**
		 * @access public
		 * @var bool
		 */
		var $Enabled;
		
		/**
		 * @access public
		 * @var short
		 * @example 
		 *		$this->Format = LOG_FORMAT_OPTIONS_NONE;
		 *		$this->Format = LOG_FORMAT_OPTIONS_ADD_DATE;
		 */
		var $Format = LOG_FORMAT_OPTIONS_ADD_DATE;
		
		/**
		 * @access public
		 * @var string
		 */
		var $LogFilePath;
		
		/**
		 * @access private
		 * @param bool $param[optional] = true
		 * @return CLog
		 */
		function CLog($param = true)
		{
		    if (!is_null($param))
		    {
		    	die(CANT_CALL_CONSTRUCTOR);
		    }		
			
			$settings =& Settings::CreateInstance();
			$this->Enabled = $settings->EnableLogging;

			$this->LogFilePath = INI_DIR.'/'.LOG_PATH.'/'.LOG_FILENAME;
			if ($this->Enabled && !is_dir(INI_DIR.'/'.LOG_PATH))
			{
				@mkdir(INI_DIR.'/'.LOG_PATH);
			}
		}
		
		/**
		 * @static
		 * @access public
		 * @return CLog
		 */
		function &CreateInstance()
		{
			static $instance;
    		if (!is_object($instance))
    		{
				$instance = new CLog(null);
    		}
    		return $instance;
		}
		
		/**
		 * @access public
		 * @param string $errorDesc
		 * @param int $line[optional] = ''
		 */
		function WriteLine($errorDesc, $line = '')
		{
			if (!$this->Enabled) 
			{
				return;
			}
			
			static $_isFirst = true;
			static $_lastTime;
			
			if ($_isFirst)
			{
				$_uri = isset($_SERVER['REQUEST_URI']) ? $_SERVER['REQUEST_URI'] : '';
				$_isFirst = false;
				$_post = (isset($_POST) && count($_POST) > 0) ? ' [POST('.count($_POST).')]' : '';
				@error_log("\r\n".'['.date('d/m/Y H:i:s').'] --- start ver.'.WMVERSION.' ---> '.$_uri.$_post."\r\n", 3, $this->LogFilePath);
				if (strlen($_post) > 0)
				{
					@error_log('['.date('d/m/Y H:i:s').'] POST > ['.implode(', ', array_keys($_POST))."]\r\n", 3, $this->LogFilePath);
				}
			}
			
			if (LOG_LINELIMIT && strlen($errorDesc) > LOG_LINELIMIT * 2)
			{
				$errorDesc = 
					substr($errorDesc, 0, LOG_LINELIMIT).
					"\r\n ----------- cut ------------ \r\n".
					substr($errorDesc, - (int) ceil(LOG_LINELIMIT/2));
			}
			
			$line = (strlen($line) > 0) ? '[line: '.$line.']' : '';
			$_newTime = CLog::getmicrotime();
			$_newSTime = (int) (CLog::getmicrosec() * 1000);  
			$_tmpTime = (int) (($_newTime - $_lastTime) * 1000);
			$_pref = '';
			if ($_lastTime && $_tmpTime > LOG_TIMERUN)
			{
				$_pref = $_tmpTime.'::';
			}
			$_lastTime = $_newTime;
			@error_log($_pref.'['.date('H:i:s', $_newTime).'.'.$_newSTime.']'.$line.' '.$errorDesc ."\r\n", 3, $this->LogFilePath);
		}
		
		/**
		 * @return float
		 */
		function getmicrotime() 
		{ 
	    	list($usec, $sec) = explode(' ', microtime()); 
	    	return ((float)$usec + (float)$sec); 
		}
		
		/**
		 * @return float
		 */
		function getmicrosec() 
		{ 
	    	$usec = explode(' ', microtime()); 
	    	return isset($usec[0]) ? (float) $usec[0] : -1; 
		}
	}
